@@ -8,7 +8,6 @@
 #include "adw-clamp-scrollable.h"
 
 #include "adw-clamp-layout.h"
-#include "adw-macros-private.h"
 #include "adw-widget-utils-private.h"
 
 /**
@@ -21,8 +20,6 @@
  *
  * The primary use case for `AdwClampScrollable` is clamping
  * [class@Gtk.ListView].
- *
- * Since: 1.0
  */
 
 enum {
@@ -257,15 +254,11 @@ adw_clamp_scrollable_class_init (AdwClampScrollableClass *klass)
    * AdwClampScrollable:child: (attributes org.gtk.Property.get=adw_clamp_scrollable_get_child org.gtk.Property.set=adw_clamp_scrollable_set_child)
    *
    * The child widget of the `AdwClampScrollable`.
-   *
-   * Since: 1.0
    */
   props[PROP_CHILD] =
-      g_param_spec_object ("child",
-                           "Child",
-                           "The child widget",
-                           GTK_TYPE_WIDGET,
-                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+    g_param_spec_object ("child", NULL, NULL,
+                         GTK_TYPE_WIDGET,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwClampScrollable:maximum-size: (attributes org.gtk.Property.get=adw_clamp_scrollable_get_maximum_size org.gtk.Property.set=adw_clamp_scrollable_set_maximum_size)
@@ -273,15 +266,11 @@ adw_clamp_scrollable_class_init (AdwClampScrollableClass *klass)
    * The maximum size allocated to the child.
    *
    * It is the width if the clamp is horizontal, or the height if it is vertical.
-   *
-   * Since: 1.0
    */
   props[PROP_MAXIMUM_SIZE] =
-      g_param_spec_int ("maximum-size",
-                        "Maximum size",
-                        "The maximum size allocated to the child",
-                        0, G_MAXINT, 600,
-                        G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+    g_param_spec_int ("maximum-size", NULL, NULL,
+                      0, G_MAXINT, 600,
+                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwClampScrollable:tightening-threshold: (attributes org.gtk.Property.get=adw_clamp_scrollable_get_tightening_threshold org.gtk.Property.set=adw_clamp_scrollable_set_tightening_threshold)
@@ -300,15 +289,11 @@ adw_clamp_scrollable_class_init (AdwClampScrollableClass *klass)
    *
    * Effectively, tightening the grip on the child before it reaches its maximum
    * size makes transitions to and from the maximum size smoother when resizing.
-   *
-   * Since: 1.0
    */
   props[PROP_TIGHTENING_THRESHOLD] =
-      g_param_spec_int ("tightening-threshold",
-                        "Tightening threshold",
-                        "The size from which the clamp will tighten its grip on the child",
-                        0, G_MAXINT, 400,
-                        G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+    g_param_spec_int ("tightening-threshold", NULL, NULL,
+                      0, G_MAXINT, 400,
+                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
@@ -347,8 +332,6 @@ adw_clamp_scrollable_buildable_init (GtkBuildableIface *iface)
  * Creates a new `AdwClampScrollable`.
  *
  * Returns: the newly created `AdwClampScrollable`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_clamp_scrollable_new (void)
@@ -363,8 +346,6 @@ adw_clamp_scrollable_new (void)
  * Gets the child widget of @self.
  *
  * Returns: (nullable) (transfer none): the child widget of @self
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_clamp_scrollable_get_child (AdwClampScrollable *self)
@@ -380,8 +361,6 @@ adw_clamp_scrollable_get_child (AdwClampScrollable *self)
  * @child: (nullable): the child widget
  *
  * Sets the child widget of @self.
- *
- * Since: 1.0
  */
 void
 adw_clamp_scrollable_set_child (AdwClampScrollable *self,
@@ -435,8 +414,6 @@ adw_clamp_scrollable_set_child (AdwClampScrollable *self,
  * Gets the maximum size allocated to the child.
  *
  * Returns: the maximum size to allocate to the child
- *
- * Since: 1.0
  */
 int
 adw_clamp_scrollable_get_maximum_size (AdwClampScrollable *self)
@@ -457,7 +434,7 @@ adw_clamp_scrollable_get_maximum_size (AdwClampScrollable *self)
  *
  * Sets the maximum size allocated to the child.
  *
- * Since: 1.0
+ * It is the width if the clamp is horizontal, or the height if it is vertical.
  */
 void
 adw_clamp_scrollable_set_maximum_size (AdwClampScrollable *self,
@@ -484,8 +461,6 @@ adw_clamp_scrollable_set_maximum_size (AdwClampScrollable *self,
  * Gets the size above which the child is clamped.
  *
  * Returns: the size above which the child is clamped
- *
- * Since: 1.0
  */
 int
 adw_clamp_scrollable_get_tightening_threshold (AdwClampScrollable *self)
@@ -506,7 +481,18 @@ adw_clamp_scrollable_get_tightening_threshold (AdwClampScrollable *self)
  *
  * Sets the size above which the child is clamped.
  *
- * Since: 1.0
+ * Starting from this size, the clamp will tighten its grip on the child, slowly
+ * allocating less and less of the available size up to the maximum allocated
+ * size. Below that threshold and below the maximum width, the child will be
+ * allocated all the available size.
+ *
+ * If the threshold is greater than the maximum size to allocate to the child,
+ * the child will be allocated all the width up to the maximum. If the threshold
+ * is lower than the minimum size to allocate to the child, that size will be
+ * used as the tightening threshold.
+ *
+ * Effectively, tightening the grip on the child before it reaches its maximum
+ * size makes transitions to and from the maximum size smoother when resizing.
  */
 void
 adw_clamp_scrollable_set_tightening_threshold (AdwClampScrollable *self,
