@@ -8,7 +8,6 @@
 #include "adw-expander-row.h"
 
 #include "adw-action-row.h"
-#include "adw-macros-private.h"
 #include "adw-widget-utils-private.h"
 
 /**
@@ -42,8 +41,6 @@
  * It contains the subnodes `row.header` for its main embedded row,
  * `list.nested` for the list it can expand, and `image.expander-row-arrow` for
  * its arrow.
- *
- * Since: 1.0
  */
 
 typedef struct
@@ -77,6 +74,8 @@ enum {
   PROP_EXPANDED,
   PROP_ENABLE_EXPANSION,
   PROP_SHOW_ENABLE_SWITCH,
+  PROP_TITLE_LINES,
+  PROP_SUBTITLE_LINES,
   LAST_PROP,
 };
 
@@ -130,6 +129,12 @@ adw_expander_row_get_property (GObject    *object,
   case PROP_SHOW_ENABLE_SWITCH:
     g_value_set_boolean (value, adw_expander_row_get_show_enable_switch (self));
     break;
+  case PROP_TITLE_LINES:
+    g_value_set_int (value, adw_expander_row_get_title_lines (self));
+    break;
+  case PROP_SUBTITLE_LINES:
+    g_value_set_int (value, adw_expander_row_get_subtitle_lines (self));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -159,6 +164,12 @@ adw_expander_row_set_property (GObject      *object,
   case PROP_SHOW_ENABLE_SWITCH:
     adw_expander_row_set_show_enable_switch (self, g_value_get_boolean (value));
     break;
+  case PROP_TITLE_LINES:
+    adw_expander_row_set_title_lines (self, g_value_get_int (value));
+    break;
+  case PROP_SUBTITLE_LINES:
+    adw_expander_row_set_subtitle_lines (self, g_value_get_int (value));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -183,8 +194,6 @@ adw_expander_row_class_init (AdwExpanderRowClass *klass)
    *
    * The subtitle is interpreted as Pango markup unless
    * [property@PreferencesRow:use-markup] is set to `FALSE`.
-   *
-   * Since: 1.0
    */
   props[PROP_SUBTITLE] =
     g_param_spec_string ("subtitle", NULL, NULL,
@@ -196,7 +205,7 @@ adw_expander_row_class_init (AdwExpanderRowClass *klass)
    *
    * The icon name for this row.
    *
-   * Since: 1.0
+   * Deprecated: 1.3: Use [method@ExpanderRow.add_prefix] to add an icon.
    */
   props[PROP_ICON_NAME] =
     g_param_spec_string ("icon-name", NULL, NULL,
@@ -207,8 +216,6 @@ adw_expander_row_class_init (AdwExpanderRowClass *klass)
    * AdwExpanderRow:expanded: (attributes org.gtk.Property.get=adw_expander_row_get_expanded org.gtk.Property.set=adw_expander_row_set_expanded)
    *
    * Whether the row is expanded.
-   *
-   * Since: 1.0
    */
   props[PROP_EXPANDED] =
     g_param_spec_boolean ("expanded", NULL, NULL,
@@ -219,8 +226,6 @@ adw_expander_row_class_init (AdwExpanderRowClass *klass)
    * AdwExpanderRow:enable-expansion: (attributes org.gtk.Property.get=adw_expander_row_get_enable_expansion org.gtk.Property.set=adw_expander_row_set_enable_expansion)
    *
    * Whether expansion is enabled.
-   *
-   * Since: 1.0
    */
   props[PROP_ENABLE_EXPANSION] =
     g_param_spec_boolean ("enable-expansion", NULL, NULL,
@@ -231,14 +236,43 @@ adw_expander_row_class_init (AdwExpanderRowClass *klass)
    * AdwExpanderRow:show-enable-switch: (attributes org.gtk.Property.get=adw_expander_row_get_show_enable_switch org.gtk.Property.set=adw_expander_row_set_show_enable_switch)
    *
    * Whether the switch enabling the expansion is visible.
-   *
-   * Since: 1.0
    */
   props[PROP_SHOW_ENABLE_SWITCH] =
     g_param_spec_boolean ("show-enable-switch", NULL, NULL,
                           FALSE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * AdwExpanderRow:title-lines: (attributes org.gtk.Property.get=adw_expander_row_get_title_lines org.gtk.Property.set=adw_expander_row_set_title_lines)
+   *
+   * The number of lines at the end of which the title label will be ellipsized.
+   *
+   * If the value is 0, the number of lines won't be limited.
+   *
+   * Since: 1.3
+   */
+  props[PROP_TITLE_LINES] =
+    g_param_spec_int ("title-lines", NULL, NULL,
+                      0, G_MAXINT,
+                      0,
+                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * AdwExpanderRow:subtitle-lines: (attributes org.gtk.Property.get=adw_expander_row_get_subtitle_lines org.gtk.Property.set=adw_expander_row_set_subtitle_lines)
+   *
+   * The number of lines at the end of which the subtitle label will be
+   * ellipsized.
+   *
+   * If the value is 0, the number of lines won't be limited.
+   *
+   * Since: 1.3
+   */
+  props[PROP_SUBTITLE_LINES] =
+    g_param_spec_int ("subtitle-lines", NULL, NULL,
+                      0, G_MAXINT,
+                      0,
+                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
   gtk_widget_class_set_template_from_resource (widget_class,
@@ -261,6 +295,8 @@ func (gpointer this) { \
 
 NOTIFY (notify_subtitle_cb, PROP_SUBTITLE);
 NOTIFY (notify_icon_name_cb, PROP_ICON_NAME);
+NOTIFY (notify_title_lines_cb, PROP_TITLE_LINES);
+NOTIFY (notify_subtitle_lines_cb, PROP_SUBTITLE_LINES);
 
 static void
 adw_expander_row_init (AdwExpanderRow *self)
@@ -276,6 +312,8 @@ adw_expander_row_init (AdwExpanderRow *self)
 
   g_signal_connect_object (priv->action_row, "notify::subtitle", G_CALLBACK (notify_subtitle_cb), self, G_CONNECT_SWAPPED);
   g_signal_connect_object (priv->action_row, "notify::icon-name", G_CALLBACK (notify_icon_name_cb), self, G_CONNECT_SWAPPED);
+  g_signal_connect_object (priv->action_row, "notify::title-lines", G_CALLBACK (notify_title_lines_cb), self, G_CONNECT_SWAPPED);
+  g_signal_connect_object (priv->action_row, "notify::subtitle-lines", G_CALLBACK (notify_subtitle_lines_cb), self, G_CONNECT_SWAPPED);
 }
 
 static void
@@ -313,8 +351,6 @@ adw_expander_row_buildable_init (GtkBuildableIface *iface)
  * Creates a new `AdwExpanderRow`.
  *
  * Returns: the newly created `AdwExpanderRow`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_expander_row_new (void)
@@ -328,8 +364,6 @@ adw_expander_row_new (void)
  * @widget: a widget
  *
  * Adds an action widget to @self.
- *
- * Since: 1.0
  */
 void
 adw_expander_row_add_action (AdwExpanderRow *self,
@@ -343,7 +377,7 @@ adw_expander_row_add_action (AdwExpanderRow *self,
   priv = adw_expander_row_get_instance_private (self);
 
   gtk_box_prepend (priv->actions, widget);
-  gtk_widget_show (GTK_WIDGET (priv->actions));
+  gtk_widget_set_visible (GTK_WIDGET (priv->actions), TRUE);
 }
 
 /**
@@ -352,8 +386,6 @@ adw_expander_row_add_action (AdwExpanderRow *self,
  * @widget: a widget
  *
  * Adds a prefix widget to @self.
- *
- * Since: 1.0
  */
 void
 adw_expander_row_add_prefix (AdwExpanderRow *self,
@@ -381,8 +413,6 @@ adw_expander_row_add_prefix (AdwExpanderRow *self,
  * Adds a widget to @self.
  *
  * The widget will appear in the expanding list below @self.
- *
- * Since: 1.0
  */
 void
 adw_expander_row_add_row (AdwExpanderRow *self,
@@ -409,8 +439,6 @@ adw_expander_row_add_row (AdwExpanderRow *self,
  * @widget: the child to be removed
  *
  * Removes a child from @self.
- *
- * Since: 1.0
  */
 void
 adw_expander_row_remove (AdwExpanderRow *self,
@@ -449,8 +477,6 @@ adw_expander_row_remove (AdwExpanderRow *self,
  * Gets the subtitle for @self.
  *
  * Returns: the subtitle for @self
- *
- * Since: 1.0
  */
 const char *
 adw_expander_row_get_subtitle (AdwExpanderRow *self)
@@ -473,8 +499,6 @@ adw_expander_row_get_subtitle (AdwExpanderRow *self)
  *
  * The subtitle is interpreted as Pango markup unless
  * [property@PreferencesRow:use-markup] is set to `FALSE`.
- *
- * Since: 1.0
  */
 void
 adw_expander_row_set_subtitle (AdwExpanderRow *self,
@@ -497,7 +521,7 @@ adw_expander_row_set_subtitle (AdwExpanderRow *self,
  *
  * Returns: (nullable): the icon name for @self
  *
- * Since: 1.0
+ * Deprecated: 1.3: Use [method@ExpanderRow.add_prefix] to add an icon.
  */
 const char *
 adw_expander_row_get_icon_name (AdwExpanderRow *self)
@@ -518,7 +542,7 @@ adw_expander_row_get_icon_name (AdwExpanderRow *self)
  *
  * Sets the icon name for @self.
  *
- * Since: 1.0
+ * Deprecated: 1.3: Use [method@ExpanderRow.add_prefix] to add an icon.
  */
 void
 adw_expander_row_set_icon_name (AdwExpanderRow *self,
@@ -540,8 +564,6 @@ adw_expander_row_set_icon_name (AdwExpanderRow *self,
  * Gets whether @self is expanded.
  *
  * Returns: whether @self is expanded
- *
- * Since: 1.0
  */
 gboolean
 adw_expander_row_get_expanded (AdwExpanderRow *self)
@@ -561,8 +583,6 @@ adw_expander_row_get_expanded (AdwExpanderRow *self)
  * @expanded: whether to expand the row
  *
  * Sets whether @self is expanded.
- *
- * Since: 1.0
  */
 void
 adw_expander_row_set_expanded (AdwExpanderRow *self,
@@ -586,6 +606,10 @@ adw_expander_row_set_expanded (AdwExpanderRow *self,
   else
     gtk_widget_unset_state_flags (GTK_WIDGET (self), GTK_STATE_FLAG_CHECKED);
 
+  gtk_accessible_update_state (GTK_ACCESSIBLE (priv->action_row),
+                               GTK_ACCESSIBLE_STATE_EXPANDED, priv->expanded,
+                               -1);
+
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_EXPANDED]);
 }
 
@@ -596,8 +620,6 @@ adw_expander_row_set_expanded (AdwExpanderRow *self,
  * Gets whether the expansion of @self is enabled.
  *
  * Returns: whether the expansion of @self is enabled.
- *
- * Since: 1.0
  */
 gboolean
 adw_expander_row_get_enable_expansion (AdwExpanderRow *self)
@@ -617,8 +639,6 @@ adw_expander_row_get_enable_expansion (AdwExpanderRow *self)
  * @enable_expansion: whether to enable the expansion
  *
  * Sets whether the expansion of @self is enabled.
- *
- * Since: 1.0
  */
 void
 adw_expander_row_set_enable_expansion (AdwExpanderRow *self,
@@ -649,8 +669,6 @@ adw_expander_row_set_enable_expansion (AdwExpanderRow *self,
  * Gets whether the switch enabling the expansion of @self is visible.
  *
  * Returns: whether the switch enabling the expansion is visible
- *
- * Since: 1.0
  */
 gboolean
 adw_expander_row_get_show_enable_switch (AdwExpanderRow *self)
@@ -670,8 +688,6 @@ adw_expander_row_get_show_enable_switch (AdwExpanderRow *self)
  * @show_enable_switch: whether to show the switch enabling the expansion
  *
  * Sets whether the switch enabling the expansion of @self is visible.
- *
- * Since: 1.0
  */
 void
 adw_expander_row_set_show_enable_switch (AdwExpanderRow *self,
@@ -692,3 +708,101 @@ adw_expander_row_set_show_enable_switch (AdwExpanderRow *self,
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SHOW_ENABLE_SWITCH]);
 }
+
+/**
+ * adw_expander_row_get_title_lines: (attributes org.gtk.Method.get_property=title-lines)
+ * @self: an expander row
+ *
+ * Gets the number of lines at the end of which the title label will be
+ * ellipsized.
+ *
+ * Returns: the number of lines at the end of which the title label will be
+ *   ellipsized
+ *
+ * Since: 1.3
+ */
+int
+adw_expander_row_get_title_lines (AdwExpanderRow *self)
+{
+  AdwExpanderRowPrivate *priv;
+
+  g_return_val_if_fail (ADW_IS_EXPANDER_ROW (self), 0);
+
+  priv = adw_expander_row_get_instance_private (self);
+
+  return adw_action_row_get_title_lines (priv->action_row);
+}
+
+/**
+ * adw_expander_row_set_title_lines: (attributes org.gtk.Method.set_property=title-lines)
+ * @self: an expander row
+ * @title_lines: the number of lines at the end of which the title label will be ellipsized
+ *
+ * Sets the number of lines at the end of which the title label will be
+ * ellipsized.
+ *
+ * If the value is 0, the number of lines won't be limited.
+ *
+ * Since: 1.3
+ */
+void
+adw_expander_row_set_title_lines (AdwExpanderRow *self,
+                                  int             title_lines)
+{
+  AdwExpanderRowPrivate *priv;
+
+  g_return_if_fail (ADW_IS_EXPANDER_ROW (self));
+
+  priv = adw_expander_row_get_instance_private (self);
+
+  adw_action_row_set_title_lines (priv->action_row, title_lines);
+}
+
+/**
+ * adw_expander_row_get_subtitle_lines: (attributes org.gtk.Method.get_property=subtitle-lines)
+ * @self: an expander row
+ *
+ * Gets the number of lines at the end of which the subtitle label will be
+ * ellipsized.
+ *
+ * Returns: the number of lines at the end of which the subtitle label will be
+ *   ellipsized
+ *
+ * Since: 1.3
+ */
+int
+adw_expander_row_get_subtitle_lines (AdwExpanderRow *self)
+{
+  AdwExpanderRowPrivate *priv;
+
+  g_return_val_if_fail (ADW_IS_EXPANDER_ROW (self), 0);
+
+  priv = adw_expander_row_get_instance_private (self);
+
+  return adw_action_row_get_subtitle_lines (priv->action_row);
+}
+
+/**
+ * adw_expander_row_set_subtitle_lines: (attributes org.gtk.Method.set_property=subtitle-lines)
+ * @self: an expander row
+ * @subtitle_lines: the number of lines at the end of which the subtitle label will be ellipsized
+ *
+ * Sets the number of lines at the end of which the subtitle label will be
+ * ellipsized.
+ *
+ * If the value is 0, the number of lines won't be limited.
+ *
+ * Since: 1.3
+ */
+void
+adw_expander_row_set_subtitle_lines (AdwExpanderRow *self,
+                                     int             subtitle_lines)
+{
+  AdwExpanderRowPrivate *priv;
+
+  g_return_if_fail (ADW_IS_EXPANDER_ROW (self));
+
+  priv = adw_expander_row_get_instance_private (self);
+
+  adw_action_row_set_subtitle_lines (priv->action_row, subtitle_lines);
+}
