@@ -7,8 +7,9 @@
 #include "config.h"
 
 #include "adw-swipe-tracker-private.h"
+
+#include "adw-marshalers.h"
 #include "adw-navigation-direction.h"
-#include "adw-macros-private.h"
 
 #include <math.h>
 
@@ -28,12 +29,6 @@
 #define DRAG_THRESHOLD_DISTANCE 16
 #define EPSILON 0.005
 
-#if GTK_CHECK_VERSION (4, 7, 0)
-#define SCROLL_MULTIPLIER 1
-#else
-#define SCROLL_MULTIPLIER 10
-#endif
-
 #define SIGN(x) ((x) > 0.0 ? 1.0 : ((x) < 0.0 ? -1.0 : 0.0))
 
 /**
@@ -49,8 +44,6 @@
  * property. If they expect to use horizontal orientation,
  * [property@SwipeTracker:reversed] can be used for supporting RTL text
  * direction.
- *
- * Since: 1.0
  */
 
 typedef enum {
@@ -291,7 +284,7 @@ find_next_point (double *points,
   guint i;
 
   for (i = 0; i < n; i++)
-    if (points[i] >= pos)
+    if (G_APPROX_VALUE (points[i], pos, DBL_EPSILON) || points[i] > pos)
       return i;
 
   return -1;
@@ -305,7 +298,7 @@ find_previous_point (double *points,
   int i;
 
   for (i = n - 1; i >= 0; i--)
-    if (points[i] <= pos)
+    if (G_APPROX_VALUE (points[i], pos, DBL_EPSILON) || points[i] < pos)
       return i;
 
   return -1;
@@ -502,8 +495,10 @@ is_in_swipe_area (AdwSwipeTracker        *self,
 
   adw_swipeable_get_swipe_area (self->swipeable, direction, is_drag, &rect);
 
-  return x >= rect.x && x < rect.x + rect.width &&
-         y >= rect.y && y < rect.y + rect.height;
+  return (G_APPROX_VALUE (x, rect.x, DBL_EPSILON) || x > rect.x) &&
+         x < rect.x + rect.width &&
+         (G_APPROX_VALUE (y, rect.y, DBL_EPSILON) || y > rect.y) &&
+         y < rect.y + rect.height;
 }
 
 static void
@@ -589,10 +584,13 @@ drag_update_cb (AdwSwipeTracker *self,
     get_range (self, &first_point, &last_point);
 
     drag_distance = sqrt (offset_x * offset_x + offset_y * offset_y);
-    is_overshooting = (offset < 0 && self->progress <= first_point) ||
-                      (offset > 0 && self->progress >= last_point);
+    is_overshooting = (offset < 0 && (G_APPROX_VALUE (self->progress, first_point, DBL_EPSILON) ||
+                                      self->progress < first_point)) ||
+                      (offset > 0 && (G_APPROX_VALUE (self->progress, last_point, DBL_EPSILON) ||
+                                      self->progress > last_point));
 
-    if (drag_distance >= DRAG_THRESHOLD_DISTANCE) {
+    if (G_APPROX_VALUE (drag_distance, DRAG_THRESHOLD_DISTANCE, DBL_EPSILON) ||
+        drag_distance > DRAG_THRESHOLD_DISTANCE) {
       double start_x, start_y;
       AdwNavigationDirection direction;
 
@@ -722,10 +720,14 @@ handle_scroll_event (AdwSwipeTracker *self,
 
     get_range (self, &first_point, &last_point);
 
-    is_overshooting = (delta < 0 && self->progress <= first_point) ||
-                      (delta > 0 && self->progress >= last_point);
+    is_overshooting = (delta < 0 &&
+                       (G_APPROX_VALUE (self->progress, first_point, DBL_EPSILON) ||
+                        self->progress < first_point)) ||
+                      (delta > 0 &&
+                       (G_APPROX_VALUE (self->progress, last_point, DBL_EPSILON) ||
+                        self->progress > last_point));
 
-    append_to_history (self, delta * SCROLL_MULTIPLIER, time);
+    append_to_history (self, delta, time);
 
     if (!is_overshooting)
       gesture_begin (self);
@@ -737,9 +739,9 @@ handle_scroll_event (AdwSwipeTracker *self,
     if (gdk_scroll_event_is_stop (event)) {
       gesture_end (self, distance, time, TRUE);
     } else {
-      append_to_history (self, delta * SCROLL_MULTIPLIER, time);
+      append_to_history (self, delta, time);
 
-      gesture_update (self, delta / distance * SCROLL_MULTIPLIER, time);
+      gesture_update (self, delta / distance, time);
       return GDK_EVENT_STOP;
     }
   }
@@ -1023,15 +1025,11 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
    * AdwSwipeTracker:swipeable: (attributes org.gtk.Property.get=adw_swipe_tracker_get_swipeable)
    *
    * The widget the swipe tracker is attached to.
-   *
-   * Since: 1.0
    */
   props[PROP_SWIPEABLE] =
-    g_param_spec_object ("swipeable",
-                         "Swipeable",
-                         "The widget the swipe tracker is attached to",
+    g_param_spec_object ("swipeable", NULL, NULL,
                          ADW_TYPE_SWIPEABLE,
-                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
   /**
    * AdwSwipeTracker:enabled: (attributes org.gtk.Property.get=adw_swipe_tracker_get_enabled org.gtk.Property.set=adw_swipe_tracker_set_enabled)
@@ -1040,15 +1038,11 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
    *
    * When it's not enabled, no events will be processed. Usually widgets will
    * want to expose this via a property.
-   *
-   * Since: 1.0
    */
   props[PROP_ENABLED] =
-    g_param_spec_boolean ("enabled",
-                          "Enabled",
-                          "Whether the swipe tracker is enabled",
+    g_param_spec_boolean ("enabled", NULL, NULL,
                           TRUE,
-                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwSwipeTracker:reversed: (attributes org.gtk.Property.get=adw_swipe_tracker_get_reversed org.gtk.Property.set=adw_swipe_tracker_set_reversed)
@@ -1057,29 +1051,21 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
    *
    * If the swipe tracker is horizontal, it can be used for supporting RTL text
    * direction.
-   *
-   * Since: 1.0
    */
   props[PROP_REVERSED] =
-    g_param_spec_boolean ("reversed",
-                          "Reversed",
-                          "Whether to reverse the swipe direction",
+    g_param_spec_boolean ("reversed", NULL, NULL,
                           FALSE,
-                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwSwipeTracker:allow-mouse-drag: (attributes org.gtk.Property.get=adw_swipe_tracker_get_allow_mouse_drag org.gtk.Property.set=adw_swipe_tracker_set_allow_mouse_drag)
    *
    * Whether to allow dragging with mouse pointer.
-   *
-   * Since: 1.0
    */
   props[PROP_ALLOW_MOUSE_DRAG] =
-    g_param_spec_boolean ("allow-mouse-drag",
-                          "Allow mouse drag",
-                          "Whether to allow dragging with mouse pointer",
+    g_param_spec_boolean ("allow-mouse-drag", NULL, NULL,
                           FALSE,
-                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwSwipeTracker:allow-long-swipes: (attributes org.gtk.Property.get=adw_swipe_tracker_get_allow_long_swipes org.gtk.Property.set=adw_swipe_tracker_set_allow_long_swipes)
@@ -1088,15 +1074,11 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
    *
    * If the value is `FALSE`, each swipe can only move to the adjacent snap
    * points.
-   *
-   * Since: 1.0
    */
   props[PROP_ALLOW_LONG_SWIPES] =
-    g_param_spec_boolean ("allow-long-swipes",
-                          "Allow long swipes",
-                          "Whether to allow swiping for more than one snap point at a time",
+    g_param_spec_boolean ("allow-long-swipes", NULL, NULL,
                           FALSE,
-                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   g_object_class_override_property (object_class,
                                     PROP_ORIENTATION,
@@ -1113,35 +1095,39 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
    *
    * The @direction value can be used to restrict the swipe to a certain
    * direction.
-   *
-   * Since: 1.0
    */
   signals[SIGNAL_PREPARE] =
     g_signal_new ("prepare",
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_FIRST,
                   0,
-                  NULL, NULL, NULL,
+                  NULL, NULL,
+                  adw_marshal_VOID__ENUM,
                   G_TYPE_NONE,
                   1,
                   ADW_TYPE_NAVIGATION_DIRECTION);
+  g_signal_set_va_marshaller (signals[SIGNAL_PREPARE],
+                              G_TYPE_FROM_CLASS (klass),
+                              adw_marshal_VOID__ENUMv);
 
   /**
    * AdwSwipeTracker::begin-swipe:
    *
    * This signal is emitted right before a swipe will be started, after the
    * drag threshold has been passed.
-   *
-   * Since: 1.0
    */
   signals[SIGNAL_BEGIN_SWIPE] =
     g_signal_new ("begin-swipe",
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_FIRST,
                   0,
-                  NULL, NULL, NULL,
+                  NULL, NULL,
+                  adw_marshal_VOID__VOID,
                   G_TYPE_NONE,
                   0);
+  g_signal_set_va_marshaller (signals[SIGNAL_BEGIN_SWIPE],
+                              G_TYPE_FROM_CLASS (klass),
+                              adw_marshal_VOID__VOIDv);
 
   /**
    * AdwSwipeTracker::update-swipe:
@@ -1149,18 +1135,20 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
    * @progress: the current animation progress value
    *
    * This signal is emitted every time the progress value changes.
-   *
-   * Since: 1.0
    */
   signals[SIGNAL_UPDATE_SWIPE] =
     g_signal_new ("update-swipe",
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_FIRST,
                   0,
-                  NULL, NULL, NULL,
+                  NULL, NULL,
+                  adw_marshal_VOID__DOUBLE,
                   G_TYPE_NONE,
                   1,
                   G_TYPE_DOUBLE);
+  g_signal_set_va_marshaller (signals[SIGNAL_UPDATE_SWIPE],
+                              G_TYPE_FROM_CLASS (klass),
+                              adw_marshal_VOID__DOUBLEv);
 
   /**
    * AdwSwipeTracker::end-swipe:
@@ -1174,18 +1162,20 @@ adw_swipe_tracker_class_init (AdwSwipeTrackerClass *klass)
    * value to @to with an animation using @velocity as the initial velocity,
    * provided in pixels per second. [class@SpringAnimation] is usually a good
    * fit for this.
-   *
-   * Since: 1.0
    */
   signals[SIGNAL_END_SWIPE] =
     g_signal_new ("end-swipe",
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_FIRST,
                   0,
-                  NULL, NULL, NULL,
+                  NULL, NULL,
+                  adw_marshal_VOID__DOUBLE_DOUBLE,
                   G_TYPE_NONE,
                   2,
                   G_TYPE_DOUBLE, G_TYPE_DOUBLE);
+  g_signal_set_va_marshaller (signals[SIGNAL_END_SWIPE],
+                              G_TYPE_FROM_CLASS (klass),
+                              adw_marshal_VOID__DOUBLE_DOUBLEv);
 }
 
 static void
@@ -1205,8 +1195,6 @@ adw_swipe_tracker_init (AdwSwipeTracker *self)
  * Creates a new `AdwSwipeTracker` for @widget.
  *
  * Returns: the newly created `AdwSwipeTracker`
- *
- * Since: 1.0
  */
 AdwSwipeTracker *
 adw_swipe_tracker_new (AdwSwipeable *swipeable)
@@ -1225,8 +1213,6 @@ adw_swipe_tracker_new (AdwSwipeable *swipeable)
  * Get the widget @self is attached to.
  *
  * Returns: (transfer none): the swipeable widget
- *
- * Since: 1.0
  */
 AdwSwipeable *
 adw_swipe_tracker_get_swipeable (AdwSwipeTracker *self)
@@ -1243,8 +1229,6 @@ adw_swipe_tracker_get_swipeable (AdwSwipeTracker *self)
  * Gets whether @self is enabled.
  *
  * Returns: whether @self is enabled
- *
- * Since: 1.0
  */
 gboolean
 adw_swipe_tracker_get_enabled (AdwSwipeTracker *self)
@@ -1261,7 +1245,8 @@ adw_swipe_tracker_get_enabled (AdwSwipeTracker *self)
  *
  * Sets whether @self is enabled.
  *
- * Since: 1.0
+ * When it's not enabled, no events will be processed. Usually widgets will want
+ * to expose this via a property.
  */
 void
 adw_swipe_tracker_set_enabled (AdwSwipeTracker *self,
@@ -1291,8 +1276,6 @@ adw_swipe_tracker_set_enabled (AdwSwipeTracker *self,
  * Gets whether @self is reversing the swipe direction.
  *
  * Returns: whether the direction is reversed
- *
- * Since: 1.0
  */
 gboolean
 adw_swipe_tracker_get_reversed (AdwSwipeTracker *self)
@@ -1309,7 +1292,8 @@ adw_swipe_tracker_get_reversed (AdwSwipeTracker *self)
  *
  * Sets whether to reverse the swipe direction.
  *
- * Since: 1.0
+ * If the swipe tracker is horizontal, it can be used for supporting RTL text
+ * direction.
  */
 void
 adw_swipe_tracker_set_reversed (AdwSwipeTracker *self,
@@ -1333,8 +1317,6 @@ adw_swipe_tracker_set_reversed (AdwSwipeTracker *self,
  * Gets whether @self can be dragged with mouse pointer.
  *
  * Returns: whether mouse dragging is allowed
- *
- * Since: 1.0
  */
 gboolean
 adw_swipe_tracker_get_allow_mouse_drag (AdwSwipeTracker *self)
@@ -1350,8 +1332,6 @@ adw_swipe_tracker_get_allow_mouse_drag (AdwSwipeTracker *self)
  * @allow_mouse_drag: whether to allow mouse dragging
  *
  * Sets whether @self can be dragged with mouse pointer.
- *
- * Since: 1.0
  */
 void
 adw_swipe_tracker_set_allow_mouse_drag (AdwSwipeTracker *self,
@@ -1378,8 +1358,6 @@ adw_swipe_tracker_set_allow_mouse_drag (AdwSwipeTracker *self,
  * Gets whether to allow swiping for more than one snap point at a time.
  *
  * Returns: whether long swipes are allowed
- *
- * Since: 1.0
  */
 gboolean
 adw_swipe_tracker_get_allow_long_swipes (AdwSwipeTracker *self)
@@ -1396,7 +1374,8 @@ adw_swipe_tracker_get_allow_long_swipes (AdwSwipeTracker *self)
  *
  * Sets whether to allow swiping for more than one snap point at a time.
  *
- * Since: 1.0
+ * If the value is `FALSE`, each swipe can only move to the adjacent snap
+ * points.
  */
 void
 adw_swipe_tracker_set_allow_long_swipes (AdwSwipeTracker *self,
@@ -1423,8 +1402,6 @@ adw_swipe_tracker_set_allow_long_swipes (AdwSwipeTracker *self,
  *
  * This can be used to adjust the current position if snap points move during
  * the gesture.
- *
- * Since: 1.0
  */
 void
 adw_swipe_tracker_shift_position (AdwSwipeTracker *self,
