@@ -7,8 +7,8 @@
 
 #include "config.h"
 
-#include "adw-macros-private.h"
 #include "adw-view-switcher-title.h"
+
 #include "adw-squeezer.h"
 #include "adw-window-title.h"
 
@@ -43,17 +43,17 @@
  *
  * ```xml
  * <object class="GtkWindow">
- *   <child type="titlebar">
+ *   <property name="titlebar">
  *     <object class="AdwHeaderBar">
  *       <property name="centering-policy">strict</property>
- *       <child type="title">
+ *       <property name="title-widget">
  *         <object class="AdwViewSwitcherTitle" id="title">
  *           <property name="stack">stack</property>
  *         </object>
- *       </child>
+ *       </property>
  *     </object>
- *   </child>
- *   <child>
+ *   </property>
+ *   <property name="child">
  *     <object class="GtkBox">
  *       <property name="orientation">vertical</property>
  *       <child>
@@ -68,15 +68,13 @@
  *         </object>
  *       </child>
  *     </object>
- *   </child>
+ *   </property>
  * </object>
  * ```
  *
  * ## CSS nodes
  *
  * `AdwViewSwitcherTitle` has a single CSS node with name `viewswitchertitle`.
- *
- * Since: 1.0
  */
 
 enum {
@@ -213,8 +211,7 @@ adw_view_switcher_title_dispose (GObject *object)
     g_clear_object (&self->pages);
   }
 
-  if (self->squeezer)
-    gtk_widget_unparent (GTK_WIDGET (self->squeezer));
+  gtk_widget_dispose_template (GTK_WIDGET (self), ADW_TYPE_VIEW_SWITCHER_TITLE);
 
   G_OBJECT_CLASS (adw_view_switcher_title_parent_class)->dispose (object);
 }
@@ -286,48 +283,36 @@ adw_view_switcher_title_class_init (AdwViewSwitcherTitleClass *klass)
    * AdwViewSwitcherTitle:stack: (attributes org.gtk.Property.get=adw_view_switcher_title_get_stack org.gtk.Property.set=adw_view_switcher_title_set_stack)
    *
    * The stack the view switcher controls.
-   *
-   * Since: 1.0
    */
   props[PROP_STACK] =
-    g_param_spec_object ("stack",
-                         "Stack",
-                         "The stack the view switcher controls",
+    g_param_spec_object ("stack", NULL, NULL,
                          ADW_TYPE_VIEW_STACK,
-                         G_PARAM_EXPLICIT_NOTIFY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwViewSwitcherTitle:title: (attributes org.gtk.Property.get=adw_view_switcher_title_get_title org.gtk.Property.set=adw_view_switcher_title_set_title)
    *
    * The title to display.
    *
-   * The title should give a user additional details. A good title should not
-   * include the application name.
-   *
-   * Since: 1.0
+   * The title typically identifies the current view or content item, and
+   * generally does not use the application name.
    */
   props[PROP_TITLE] =
-    g_param_spec_string ("title",
-                         "Title",
-                         "The title to display",
+    g_param_spec_string ("title", NULL, NULL,
                          "",
-                         G_PARAM_EXPLICIT_NOTIFY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwViewSwitcherTitle:subtitle: (attributes org.gtk.Property.get=adw_view_switcher_title_get_subtitle org.gtk.Property.set=adw_view_switcher_title_set_subtitle)
    *
    * The subtitle to display.
    *
-   * The subtitle should give a user additional details.
-   *
-   * Since: 1.0
+   * The subtitle should give the user additional details.
    */
   props[PROP_SUBTITLE] =
-    g_param_spec_string ("subtitle",
-                         "Subtitle",
-                         "The subtitle to display",
+    g_param_spec_string ("subtitle", NULL, NULL,
                          "",
-                         G_PARAM_EXPLICIT_NOTIFY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwViewSwitcherTitle:view-switcher-enabled: (attributes org.gtk.Property.get=adw_view_switcher_title_get_view_switcher_enabled org.gtk.Property.set=adw_view_switcher_title_set_view_switcher_enabled)
@@ -340,15 +325,11 @@ adw_view_switcher_title_class_init (AdwViewSwitcherTitleClass *klass)
    *
    * This can be used e.g. to ensure the view switcher is hidden below a certain
    * window width, or any other constraint you find suitable.
-   *
-   * Since: 1.0
    */
   props[PROP_VIEW_SWITCHER_ENABLED] =
-    g_param_spec_boolean ("view-switcher-enabled",
-                         "View switcher enabled",
-                         "Whether the view switcher is enabled",
+    g_param_spec_boolean ("view-switcher-enabled", NULL, NULL,
                          TRUE,
-                         G_PARAM_EXPLICIT_NOTIFY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwViewSwitcherTitle:title-visible: (attributes org.gtk.Property.get=adw_view_switcher_title_get_title_visible)
@@ -357,15 +338,11 @@ adw_view_switcher_title_class_init (AdwViewSwitcherTitleClass *klass)
    *
    * If the title is visible, it means the view switcher is hidden an it may be
    * wanted to show an alternative switcher, e.g. a [class@ViewSwitcherBar].
-   *
-   * Since: 1.0
    */
   props[PROP_TITLE_VISIBLE] =
-    g_param_spec_boolean ("title-visible",
-                         "Title visible",
-                         "Whether the title is currently visible",
+    g_param_spec_boolean ("title-visible", NULL, NULL,
                          TRUE,
-                         G_PARAM_EXPLICIT_NOTIFY | G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
@@ -400,8 +377,6 @@ adw_view_switcher_title_init (AdwViewSwitcherTitle *self)
  * Creates a new `AdwViewSwitcherTitle`.
  *
  * Returns: the newly created `AdwViewSwitcherTitle`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_view_switcher_title_new (void)
@@ -416,8 +391,6 @@ adw_view_switcher_title_new (void)
  * Gets the stack controlled by @self.
  *
  * Returns: (nullable) (transfer none): the stack
- *
- * Since: 1.0
  */
 AdwViewStack *
 adw_view_switcher_title_get_stack (AdwViewSwitcherTitle *self)
@@ -433,8 +406,6 @@ adw_view_switcher_title_get_stack (AdwViewSwitcherTitle *self)
  * @stack: (nullable): a stack
  *
  * Sets the stack controlled by @self.
- *
- * Since: 1.0
  */
 void
 adw_view_switcher_title_set_stack (AdwViewSwitcherTitle *self,
@@ -476,8 +447,6 @@ adw_view_switcher_title_set_stack (AdwViewSwitcherTitle *self,
  * Gets the title of @self.
  *
  * Returns: the title
- *
- * Since: 1.0
  */
 const char *
 adw_view_switcher_title_get_title (AdwViewSwitcherTitle *self)
@@ -494,7 +463,8 @@ adw_view_switcher_title_get_title (AdwViewSwitcherTitle *self)
  *
  * Sets the title of @self.
  *
- * Since: 1.0
+ * The title typically identifies the current view or content item, and
+ * generally does not use the application name.
  */
 void
 adw_view_switcher_title_set_title (AdwViewSwitcherTitle *self,
@@ -517,8 +487,6 @@ adw_view_switcher_title_set_title (AdwViewSwitcherTitle *self,
  * Gets the subtitle of @self.
  *
  * Returns: the subtitle
- *
- * Since: 1.0
  */
 const char *
 adw_view_switcher_title_get_subtitle (AdwViewSwitcherTitle *self)
@@ -535,7 +503,7 @@ adw_view_switcher_title_get_subtitle (AdwViewSwitcherTitle *self)
  *
  * Sets the subtitle of @self.
  *
- * Since: 1.0
+ * The subtitle should give the user additional details.
  */
 void
 adw_view_switcher_title_set_subtitle (AdwViewSwitcherTitle *self,
@@ -558,8 +526,6 @@ adw_view_switcher_title_set_subtitle (AdwViewSwitcherTitle *self,
  * Gets whether @self's view switcher is enabled.
  *
  * Returns: whether the view switcher is enabled
- *
- * Since: 1.0
  */
 gboolean
 adw_view_switcher_title_get_view_switcher_enabled (AdwViewSwitcherTitle *self)
@@ -576,7 +542,12 @@ adw_view_switcher_title_get_view_switcher_enabled (AdwViewSwitcherTitle *self)
  *
  * Sets whether @self's view switcher is enabled.
  *
- * Since: 1.0
+ * If it is disabled, the title will be displayed instead. This allows to
+ * programmatically hide the view switcher even if it fits in the available
+ * space.
+ *
+ * This can be used e.g. to ensure the view switcher is hidden below a certain
+ * window width, or any other constraint you find suitable.
  */
 void
 adw_view_switcher_title_set_view_switcher_enabled (AdwViewSwitcherTitle *self,
@@ -601,9 +572,10 @@ adw_view_switcher_title_set_view_switcher_enabled (AdwViewSwitcherTitle *self,
  *
  * Gets whether the title of @self is currently visible.
  *
- * Returns: whether the title of @self is currently visible
+ * If the title is visible, it means the view switcher is hidden an it may be
+ * wanted to show an alternative switcher, e.g. a [class@ViewSwitcherBar].
  *
- * Since: 1.0
+ * Returns: whether the title of @self is currently visible
  */
 gboolean
 adw_view_switcher_title_get_title_visible (AdwViewSwitcherTitle *self)
